<?php
/**
 * Plugin Name: Tweet Lovers
 * Plugin URI: http://www.qurl.nl/tweet-lovers/
 * Description: Tweet Lovers shows the Twitter profile pictures of the ones you are following or your followers in a widget.
 * Author: Jacco
 * Version: 1.0.1
 * Author URI: http://www.qurl.nl/
 * Tags: widget, twitter, tweet, follower, following, fans, friends, lovers
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Released under the GPL v.2, http://www.gnu.org/copyleft/gpl.html
 *
 * @version $Id: tweet-lovers.php 296169 2010-10-03 12:16:03Z qurl $
 */

	// Constants
	define('TWL_DATA_TYPE', 'json');
	define('TWL_PLUGIN_URL', WP_PLUGIN_URL . '/' . str_replace( basename(__FILE__), '', plugin_basename(__FILE__) ) );
	define('TWL_TWITTER_BASE_URL', 'http://twitter.com/');
	define('TWL_TWITTER_LIMIT', 100);
	define('TWL_TWITTER_REST', 'http://api.twitter.com/1/statuses/');
	define('TWL_TWITTER_SEARCH', 'http://search.twitter.com/search.json?q=@');
	define('TWL_REST_FOLLOWERS', TWL_TWITTER_REST . 'followers/');
	define('TWL_REST_FRIENDS', TWL_TWITTER_REST . 'friends/');
	define('TWL_TITLE_FOLLOWERS', 'My Twitter followers');
	define('TWL_TITLE_FRIENDS', 'I\'m following');
	define('TWL_TIME_LIMIT', 900);	// 15 minutes
	define('TWL_VERSION', '1.0.1');

	// TWL Functions
	/**
	 * checkUser() Checks for valid Twitter username
	 *
	 * @param string $username The username to check
	 * @return boolean
	 * @since 0.2
	 */
	function checkUser($username) {
		$url = TWL_TWITTER_SEARCH . strtolower($username);
		$data = wp_remote_fopen($url);

		if ( $data && ! empty($data) ) {
			$data = json_decode($data);

			if ( $data->max_id == '-1' ) {
				return FALSE;
			} else {
				return TRUE;
			}
		} else {
			return FALSE;
		}
	}

	/**
	 * createURL() Creates REST URL
	 *
	 * @param string $type Type (friends or followers)
	 * @return string URL or boolean FALSE when invalid
	 * @since 0.2
	 */
	function createURL($type) {
		$user = get_option('twl_twitter_username');

		if ( $user && ! empty($user) ) {
			$url_fld = 'twl_rest_' . $type;
			$const_name = strtoupper($url_fld);

			$url  = constant($const_name);
			$url .= $user . '.' . TWL_DATA_TYPE;
			return $url;
		} else {
			return FALSE;
		}
	}

	/**
	 * createWidget() Outputs widget
	 *
	 * @param string $args Widget args
	 * @param string $type Type (friends or followers)
	 * @since 0.1
	 */
	function createWidget($args, $type) {
		extract($args);
		$limit = get_option('twl_limit');
		$image_size = get_option('twl_image_size');

		$title_fld = 'twl_title_' . $type;
		$title = get_option($title_fld);
		if ( empty($title) ) {
			$const_name = strtoupper($title_fld);
			$title = constant($const_name);
		}

		$data = getData($type);

		if (! $limit || count($data) < $limit ) {
			$limit = count($data);
		}

		if ( empty($image_size) ) {
			$image_size = '48';
		}

		echo $before_widget;
		echo $before_title . $title. $after_title;

		echo '<style type="text/css"> #tweet-lovers img { margin : 3px; margin-bottom: 0px; } </style>';
		echo '<div id="tweet-lovers">';
		for ( $i = 0; $i < $limit; $i++ ) {
			$twl = $data[$i];
			echo '<a rel="nofollow" href="' . TWL_TWITTER_BASE_URL . $twl['screen_name'] . '" target="_blank" title="' . $twl['screen_name'] . '"><img src="' . $twl['image'] . '" width="' . $image_size . '" height="' . $image_size . '" alt="' . $twl['screen_name'] . '" /></a>';
		}
		echo '</div>';

		echo '<div style="' . ( $type == 'followers' && get_option('twl_followme') ? 'height:48px;line-height:48px;' : '' ) . '">';
		if ( $type == 'followers' && get_option('twl_followme') ) {
			echo '<a href="' . TWL_TWITTER_BASE_URL . get_option('twl_twitter_username') . '" target="_blank" title="Follow me on Twitter"><img src="' . TWL_PLUGIN_URL . 'twitter_followme.png" width="48" height="48" align="right" alt="Follow me" /></a>';
		}
		if ( get_option('twl_credit')) {
			echo '<div style="position:relative;top:25%;font-size:0.8em;margin-right:48px;">Powered by <a href="http://www.qurl.nl/tweet-lovers/" target="_blank">Tweet Lovers</a></div>';
		}
		echo '</div>';

		echo $after_widget;
	}

	/**
	 * getData() Retrieves data from Twitter API or cache
	 *
	 * @param string $type Type (friends or followers)
	 * @return array
	 * @since 0.1
	 */
	function getData($type) {
		$lastrun_fld = 'twl_lastrun_' . $type;
		$data_fld = 'twl_data_' . $type;

		$lastrun = get_option($lastrun_fld);
		if ( time() - $lastrun < TWL_TIME_LIMIT ) {
			$twl = get_option($data_fld);
		} else {
			$twl = array();
			$url = createURL($type);
			if ( $url ) {
				$data = wp_remote_fopen($url);
				if ( $data && ! empty($data) ) {
					// Only JSON for now
					switch( TWL_DATA_TYPE ) {
						case 'json':
							$data = json_decode($data);
							foreach ( $data as $f ) {
								$twl[ ] = array(
														'screen_name' => $f->screen_name,
														'image' 			=> $f->profile_image_url
													);
							}
							break;
					} // switch
					update_option($lastrun_fld, time());
					update_option($data_fld, $twl);
				} else {
					$twl = get_option($data_fld);
					if (! $twl ) {
						$twl = array();
					}
				}
			}
		}
		return $twl;
	}

	/**
	 * widgetControl() Saves and output Widget control
	 *
	 * @param string $type Type (friends or followers)
	 * @since 0.1
	 */
	function widgetControl($type) {
		$title_fld = 'twl_title_' . $type;
		$size = array('48', '40', '32', '24', '16');

		if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			// check username
			$user = get_option('twl_twitter_username');
			if (! empty($_POST['twl_twitter_username']) && $user != strtolower($_POST['twl_twitter_username']) ) {
				$check = checkUser($_POST['twl_twitter_username']);
				if ( $check ) {
					update_option('twl_twitter_username', strtolower($_POST['twl_twitter_username']));

					delete_option('twl_data_followers');
					delete_option('twl_data_friends');
					delete_option('twl_lastrun_followers');
					delete_option('twl_lastrun_friends');
				}
			}

			// limit
			(int) $limit = $_POST['twl_limit'];
			if ( $limit > TWL_TWITTER_LIMIT ) {
				$limit = TWL_TWITTER_LIMIT;
			}
			update_option('twl_limit', $limit);

			// followme
			if ( isset($_POST['twl_followme']) ) {
				if ( $_POST['twl_followme'] == 'on' ) {
					update_option('twl_followme', TRUE);
				} else {
					update_option('twl_followme', FALSE);
				}
			}

			// credit
			if ( $_POST['twl_credit'] == 'on' ) {
				update_option('twl_credit', TRUE);
			} else {
				update_option('twl_credit', FALSE);
			}

			update_option($title_fld, $_POST[$title_fld]);
			update_option('twl_image_size', $_POST['twl_image_size']);
		}

		_e('Widget title:');
		echo '<input class="widefat" id="' . $title_fld . '" name="' . $title_fld . '" type="text" value="' . get_option($title_fld) . '" />';
		echo '<br /><br />';

		echo 'Twitter Username';
		echo '<input class="widefat" id="twl_twitter_username" name="twl_twitter_username" type="text" value="' . get_option('twl_twitter_username') . '" />';
		echo '<br /><br />';

		echo 'Image size ';
		echo '<select name="twl_image_size">';
		foreach ( $size as $s ) {
			echo '<option value="' . $s . '" ' . ( get_option('twl_image_size') == $s ? 'selected' : '' ) . '>' . $s . '</option>';
		}
		echo '</select>';
		echo '<br /><br />';

		echo 'Number of images to display:<br />';
		echo '<input type="text" id="twl_limit" name="twl_limit" size="3" value="' . get_option('twl_limit') . '" />  (max 100)';
		echo '<br /><br />';

		if ( $type == 'followers' ) {
			echo '<input type="checkbox" id="twl_followme" name="twl_followme" ' . ( get_option('twl_followme') ? 'checked=checked' : '' ) . ' /> <label for="twl_followme">Add Twitter Follow Me button</label><br />';
		}
		echo '<input type="checkbox" id="twl_credit_' . $type . '" name="twl_credit" ' . ( get_option('twl_credit') ? 'checked=checked' : '' ) . ' /> <label for="twl_credit_' . $type . '">Give credits to Tweet Lovers</label>';
		echo '<br /><br />';
	}

	// WP hook functions
	function twl_control_followers() {
		widgetControl('followers');
	}

	function twl_control_friends() {
		widgetControl('friends');
	}

	function twl_widget_followers($args) {
		createWidget($args, 'followers');
	}

	function twl_widget_friends($args) {
		createWidget($args, 'friends');
	}

	function twl_init() {
		// Followers
		register_widget_control('Tweet Lovers Followers', 'twl_control_followers');
		register_sidebar_widget('Tweet Lovers Followers', 'twl_widget_followers');

		// Friends
		register_widget_control('Tweet Lovers Friends', 'twl_control_friends');
		register_sidebar_widget('Tweet Lovers Friends', 'twl_widget_friends');
	}

	function twl_install() {
		// refuse to activate on PHP < 5
		if ( version_compare(PHP_VERSION, '5.0.0', '<') ) {
			$plugin = plugin_basename(__FILE__);
			deactivate_plugins($plugin);
		} else {
			if ( get_option('twl_version') === FALSE ) {
				update_option('twl_credit', TRUE);
			}
			update_option('twl_version', TWL_VERSION);
		}
	}

	// WP hook action
	add_action('init', 'twl_init');
	register_activation_hook(__FILE__, 'twl_install');
?>